@@ -16,6 +16,7 @@ import numpy
 from matplotlib import pylab as plt
 from matplotlib.widgets import Slider, Button, RadioButtons
 from matplotlib import cm
+import time as _time
 
 import astropy.wcs as wcs
 from astropy import table
@@ -25,6 +26,12 @@ from astropy.io import fits
 from astroquery.simbad import Simbad
 import copy
 
+# NOTE GW, optional
+try:
+    import ligo.skymap.plot
+except ModuleNotFoundError:
+    pass 
+
 import logging
 
 logger = logging.getLogger("oda_api.plot_tools")
@@ -43,12 +50,46 @@ class OdaProduct(object):
 
 class OdaImage(OdaProduct):
 
+    def get_image_for_gallery(self, data=None, meta=None, header=None, sources=None,
+             levels=None, cmap=cm.gist_earth, unit_ID=4, det_sigma=3):
+        plt = self.build_fig(data=data, meta=meta, header=header, sources=sources,
+                              levels=levels, cmap=cmap, unit_ID=unit_ID, det_sigma=det_sigma, sliders=False)
+
+        request_time = _time.time()
+        pic_name = str(request_time) + '_image.png'
+
+        plt.savefig(pic_name)
+
+        return pic_name
+
     def show(self, data=None, meta=None, header=None, sources=None,
+             levels=None, cmap=cm.gist_earth, unit_ID=4, det_sigma=3, sliders=True):
+      
+        """
+        OdaImage.show
+        :param data: ODA data products, takes from class initialisation by default
+        :param meta: ODA data products metadata, takes from class initialisation by default
+        :param header: ODA data product image header, takes from class initialisation by default
+        :param sources: ODA catalog table, takes from class initialisation by default
+        :param levels: levels for contour plot, default is  numpy.linspace(1, 10, 10)
+        :param cmap: colormap default is cm.gist_earth,
+        :param unit_ID: the unit to plot image default is 4
+        :param det_sigma: limit detection sigma to lot from catalog, note that
+        :param sliders: plot sliders, set to false to upload images in gallery.
+        :return: matplotlib figure instance
+        """
+
+        plt = self.build_fig(data=data, meta=meta, header=header, sources=sources,
+                              levels=levels, cmap=cmap, unit_ID=unit_ID, det_sigma=det_sigma, sliders=sliders)
+
+        plt.show()
+
+    def build_fig(self, data=None, meta=None, header=None, sources=None,
              levels=None, cmap=cm.gist_earth,
-             unit_ID=4, det_sigma=3):
+             unit_ID=4, det_sigma=3, sliders=True):
 
         if levels is None:
-            levels = numpy.linspace(1, 10, 10) 
+            levels = numpy.linspace(1, 10, 10)
 
         if data is None:
             data = self.data.mosaic_image_0_mosaic.data_unit[unit_ID].data
@@ -78,13 +119,13 @@ class OdaImage(OdaProduct):
         if numpy.abs(ra.max() - 360.0) < 0.1 and numpy.abs(ra.min()) < 0.1:
             zero_crossing = True
             ind_ra = ra > 180.
-            ra[ind_ra] -= 360.            
+            ra[ind_ra] -= 360.
             ind_sort = numpy.argsort(ra, axis=-1)
             ra = numpy.take_along_axis(ra, ind_sort, axis=-1)
             data = numpy.take_along_axis(data, ind_sort, axis=-1)
-                
+
         self.cs = plt.contourf(ra, dec, data, cmap=cmap, levels=levels,
-                          extend="both", zorder=0)
+                               extend="both", zorder=0)
         self.cs.cmap.set_under('k')
         self.cs.set_clim(numpy.min(levels), numpy.max(levels))
 
@@ -157,29 +198,30 @@ class OdaImage(OdaProduct):
         plt.xlabel("RA")
         plt.ylabel("Dec")
 
-        #Nice to have : slider
-        cmin = plt.axes([0.85, 0.05, 0.02, 0.4])
-        cmax = plt.axes([0.85, 0.55, 0.02, 0.4])
-
-        data_min = data[numpy.isfinite(data)].min()
-        data_max = data[numpy.isfinite(data)].max()
-
-        self.smin = Slider(cmin, 'Min',  data_min, data_max, valinit=1., orientation='vertical')
-        self.smax = Slider(cmax, 'Max', data_min, data_max, valinit=10., orientation='vertical')
-        self.smin.on_changed(self.update)
-        self.smax.on_changed(self.update)
-
-        plt.show()
+        if sliders:
+            # Nice to have : slider
+            cmin = plt.axes([0.85, 0.05, 0.02, 0.4])
+            cmax = plt.axes([0.85, 0.55, 0.02, 0.4])
+            data_min = data[numpy.isfinite(data)].min()
+            data_max = data[numpy.isfinite(data)].max()
+            self.smin = Slider(cmin, 'Min', data_min, data_max, valinit=1., orientation='vertical')
+            self.smax = Slider(cmax, 'Max', data_min, data_max, valinit=10., orientation='vertical')
+            self.smin.on_changed(self.update)
+            self.smax.on_changed(self.update)
 
         return fig
 
+
     def update(self, x):
         if self.smin.val < self.smax.val:
             self.cs.set_clim(self.smin.val, self.smax.val)
 
 
     def write_fits(self, file_prefix=''):
-        self.data.mosaic_image_0_mosaic.write_fits_file(f'{file_prefix}mosaic.fits', overwrite=True)
+        file_fn = f'{file_prefix}mosaic.fits'
+
+        self.data.mosaic_image_0_mosaic.write_fits_file(file_fn, overwrite=True)
+        return file_fn
     
     
     def extract_catalog_from_image(self, include_new_sources=False, det_sigma=5, objects_of_interest=[],
@@ -360,8 +402,28 @@ class OdaLightCurve(OdaProduct):
             self.logger.debug('Computed time bin from TIMEDEL')
 
         return x[ind], dt_lc[ind], y[ind], dy[ind], e_min, e_max
-    
-    def show(self,  in_source_name='', systematic_fraction=0, ng_sig_limit=0, find_excesses=False):
+
+    def get_image_for_gallery(self, in_source_name='', systematic_fraction=0, ng_sig_limit=0, find_excesses=False):
+        plts = self.build_fig(in_source_name=in_source_name, systematic_fraction=systematic_fraction,
+                             ng_sig_limit=ng_sig_limit, find_excesses=find_excesses)
+
+        request_time = _time.time()
+        pic_name = str(request_time) + '_image.png'
+
+        if len(plts) == 1:
+            plts[0].savefig(pic_name)
+
+        return pic_name
+
+    def show(self, in_source_name='', systematic_fraction=0, ng_sig_limit=0, find_excesses=False):
+
+        plt = self.build_fig(in_source_name=in_source_name, systematic_fraction=systematic_fraction,
+                             ng_sig_limit=ng_sig_limit, find_excesses=find_excesses)
+
+        for p in plt:
+            p.show()
+
+    def build_fig(self,  in_source_name='', systematic_fraction=0, ng_sig_limit=0, find_excesses=False):
         #if ng_sig_limit <1 does not plot range
         combined_lc = self.data
         from scipy import stats
@@ -371,6 +433,8 @@ class OdaLightCurve(OdaProduct):
         else:
             source_names = [in_source_name]
 
+        figs = []
+
         for source_name in source_names:
             x, dx, y, dy, e_min, e_max = self.get_lc(source_name, systematic_fraction)
             if x is None:
@@ -381,7 +445,7 @@ class OdaLightCurve(OdaProduct):
 
             std_dev = numpy.std(y)
 
-            fig = plt.figure()
+            figs.append(plt.figure())
             _ = plt.errorbar(x, y, xerr=dx, yerr=dy, marker='o', capsize=0, linestyle='', label='Lightcurve')
             _ = plt.axhline(meany, color='green', linewidth=3)
             _ = plt.xlabel('Time [IJD]')
@@ -431,7 +495,7 @@ class OdaLightCurve(OdaProduct):
 
                             old_time = x[j]
 
-        return fig
+        return figs
 
 
     @staticmethod
@@ -525,8 +589,29 @@ class OdaSpectrum(OdaProduct):
 
         return specprod
 
+    def get_image_for_gallery(self, in_source_name='', systematic_fraction=0, xlim=[]):
+        pic_name = None
+        plt = self.build_fig(in_source_name=in_source_name, systematic_fraction=systematic_fraction,
+                             xlim=xlim)
+
+        if plt is not None:
+            request_time = _time.time()
+            pic_name = str(request_time) + '_image.png'
+
+            plt.savefig(pic_name)
+
+        return pic_name
+
     def show(self, in_source_name='', systematic_fraction=0, xlim=[]):
 
+        plt = self.build_fig(in_source_name=in_source_name, systematic_fraction=systematic_fraction,
+                             xlim=xlim)
+
+        if plt is not None:
+            plt.show()
+
+    def build_fig(self, in_source_name='', systematic_fraction=0, xlim=[]):
+
         if in_source_name == '':
             self.show_spectral_products()
             return
@@ -644,3 +729,93 @@ class OdaSpectrum(OdaProduct):
 
         return spec_fn, tstart, tstop, exposure
 
+class OdaGWContours(OdaProduct):
+    
+    @staticmethod
+    def _plot_single_contour(contour_coords, ax, color='r'):
+        coords = numpy.array(contour_coords)
+        try:
+            ax.plot(coords[:,0], coords[:,1], '-', transform=ax.get_transform('world'), color = color)
+        except TypeError:
+            ax.plot(coords[:,0], coords[:,1], '-', transform=ax.get_transform(), color = color)
+
+    @staticmethod
+    def _plot_contour_list(contour_list, ax, color=None):
+        kwargs = {}
+        if color is not None:
+            kwargs['color'] = color
+        for contour_coords in contour_list:
+            OdaGWContours._plot_single_contour(contour_coords, ax, **kwargs)
+    
+    def plot_event_contours(self, event, legend=True, name_in_legend=True, colors = [], ax = None):
+        if ax is None:
+            ax = plt.axes(projection='astro hours mollweide')
+        
+        if not colors:
+            prop_cycle = plt.rcParams['axes.prop_cycle']
+            colors = prop_cycle.by_key()['color']
+        
+        lpr = []
+        names = []
+        if event in self.data.contours.keys():
+            for i in range(len(self.data.contours[event].levels)):
+                color = colors[i%len(colors)]
+                OdaGWContours._plot_contour_list(self.data.contours[event].contours[i], ax, color)
+                lpr.append(plt.Rectangle((0, 0), 1, 1, fc = color))
+                names.append(f"{self.data.contours[event].name+' ' if name_in_legend else ''}{self.data.contours[event].levels[i]}%")
+        else:
+            raise ValueError(f'Wrong event name: {event}')
+        
+        if legend is True:
+            ax.legend(lpr, names)
+            
+    def plot_contours(self, legend=True, colors = [], ax = None):
+        if ax is None:
+            ax = plt.axes(projection='astro hours mollweide')
+        
+        if not colors:
+            prop_cycle = plt.rcParams['axes.prop_cycle']
+            colors = prop_cycle.by_key()['color']
+        
+        lpr = []
+        names = []
+        i = 0
+        for event, data in self.data.contours.items():
+            color = colors[i%len(colors)]
+            self.plot_event_contours(event, ax = ax, colors = [color], legend = False)
+            i+=1
+            lpr.append(plt.Rectangle((0, 0), 1, 1, fc = color))
+            names.append(event)
+
+        if legend is True:
+            ax.legend(lpr, names, numpoints=1, bbox_to_anchor=(1.05, 1), loc='upper left')            
+        
+    def show(self, event_name = None):
+        fig = self.build_fig(event_name=event_name)
+
+        if fig is not None:
+            fig.show()
+
+    def get_image_for_gallery(self, event_name=None):
+        pic_name = None
+        fig = self.build_fig(event_name=event_name)
+
+        if fig is not None:
+            request_time = _time.time()
+            pic_name = str(request_time) + '_image.png'
+
+            fig.savefig(pic_name)
+
+        return pic_name
+
+    def build_fig(self, event_name = None):
+        fig = plt.figure()
+        if event_name is None:
+            self.plot_contours()
+        else:
+            self.plot_event_contours(event_name)
+        return fig
+
+    # TODO can an implementation of this method provided?
+    def write_fits(self):
+        raise NotImplementedError
